library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.lab5_pack.all;

entity controller is
port (
    Op : in std_logic_vector(NOPC-1 downto 0);
    Funct : in std_logic_vector(NALU-1 downto 0);
    RegWrite : out std_logic;
    MemtoReg : out std_logic;
    MemWrite : out std_logic;
    Branch : out std_logic;
    ALUControl : out std_logic_vector(NALU-1 downto 0);
    ALUSrc : out std_logic;
    RegDst : out std_logic;
    Jump : out std_logic
);
end controller;

architecture Behavioral of controller is

signal func_mod : std_logic_vector(NALU-1 downto 0);
signal op_imm : std_logic_vector(NALU-1 downto 0);

begin

    -- ALUSrc is 0 for R-type, branch or jump; 1 for other instructions
    ALUSrc <= '0' when Op(5 downto 3) = "000" else '1';
    -- MemtoReg is 1 for lw; 0 for other instructions
    MemtoReg <= '1' when Op(5 downto 0) = "100011" else '0';
    -- RegWrite is 1 for R-type, I-type or lw; 0 for other instructions
    regwrite <= '1' when (Op(5 downto 0) = "000000"
                          and (--(inst(5 downto 3) = "000" -- Shifts
                                --and inst(1 downto 0) /= "01") -- Exclude invalid
                               --or inst(5 downto 0) = "001001" -- jalr
                               --or
                                Funct(5 downto 3) = "100" -- add/sub/logic
                               or Funct(5 downto 1) = "10101")) -- slt/sltu
                         --or inst(31 downto 26) = "000011" -- jal
                         or Op(5 downto 3) = "001" -- Immediate
                         or --(
                            Op(5 downto 0) = "100011" -- Load
                            -- and inst(27 downto 26) /= "10" -- Exclude lwl/lwr
                            -- and inst(28 downto 26) /= "111") -- Exclude invalid
           else '0';
    -- MemWrite is 1 for sw; 0 for other instructions
    MemWrite <= '1' when Op(5 downto 0) = "101011" else '0';
    -- RegDst is 0 for I-type and lw; 1 for other instructions
    RegDst <= '0' when (Op(5 downto 3) = "001" or Op(5 downto 0) = "100011") else '1';

    with Op(2 downto 0) select
        op_imm <= "100000" when "000", -- addi
                  "100001" when "001", -- addiu
                  "101000" when "010", -- slti
                  "101001" when "011", -- sltiu
                  "100100" when "100", -- andi
                  "100101" when "101", -- ori
                  "100110" when "110", -- xori
                  (others => '0') when others;

	with Funct select
		func_mod <= "101000" when "101010",
					"101001" when "101011",
					Funct when others;

    ALUControl <= func_mod when Op(5 downto 0) = "000000" -- R-type
                else op_imm when Op(5 downto 3) = "001" -- I-type
                else "100000" when Op(5 downto 0) = "100011" or Op(5 downto 0) = "101011" -- lw/sw
                else "111100" when Op(5 downto 0) = "000100" -- beq
                else "111101" when Op(5 downto 0) = "000101" -- bne
                else "111110" when Op(5 downto 0) = "000110" -- blez
                else "111111" when Op(5 downto 0) = "000111" -- bgtz
                else (others => '0');
                      
    Branch <= '1' when Op(5 downto 2) = "0001" -- beq/bne/blez/bgtz
         else '0'; -- bltz/bgez not supported as control doesn't take rt

    Jump <= '1' when Op(5 downto 0) = "000010" else '0';

end Behavioral;
