-- regfile.vhd
-- Lab 4
-- Team CART
-- CSE 132L
-- The register file block has 32 registers, each of width 32 bits. It
-- has 2 read ports, and one write port. The read ports operate asynchronously,
-- while the write port operates synchronously. The register file has a
-- write enable port, as well as a synchronous reset input, which if enabled,
-- resets all registers to 0 on the rising edge of the clock.

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.lab5_pack.all;

entity regfile is
port (
    clk : in std_logic;
    rst_s : in std_logic; -- synchronous reset
    we : in std_logic; -- write enable
    raddr_1 : in std_logic_vector (NREG-1 downto 0); -- read address 1
    raddr_2 : in std_logic_vector (NREG-1 downto 0); -- read address 2
    waddr : in std_logic_vector (NREG-1 downto 0); -- write address
    rdata_1 : out std_logic_vector (NBIT-1 downto 0); -- read data 1
    rdata_2 : out std_logic_vector (NBIT-1 downto 0); -- read data 2
    wdata : in std_logic_vector (NBIT-1 downto 0) -- write data
);
end regfile;

architecture Behavioral of regfile is

type reg_array is array ((2**NREG)-1 downto 0) of std_logic_vector (NBIT-1 downto 0);
signal reg_file : reg_array := (others => (others => '0'));

begin

    rdata_1 <= reg_file(to_integer(unsigned(raddr_1)));
    rdata_2 <= reg_file(to_integer(unsigned(raddr_2)));

    Process (clk)
    variable wa : integer := 0;
    begin
        if clk'event and clk = '0' then
            if rst_s = '1' then
                reg_file <= (others => (others => '0'));
            elsif we = '1' then
                wa := to_integer(unsigned(waddr));
                reg_file(wa) <= wdata;
            end if;
        end if;
    end Process;
    
end Behavioral;
