-- controller.vhd
-- Lab 2
-- Team CART
-- CSE 132L
-- The controller block is responsible for setting the correct value
-- for the select lines of the following multiplexers in the datapath:
--   Choosing the write address for the register file
--   Choosing the write data for the register file
--   Choosing the second operand of the ALU
-- The controller also determines whether the write enable for the
-- register file and the data memory should be enabled or not, based
-- on the instruction, as determined by the func and op fields of the
-- instruction. Additionally, it determines the correct function code
-- for the ALU from func and op.
-- Another output of the controller determines whether the instruction
-- is a branch instruction or not, but this is currently disabled for
-- this lab.
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use work.lab2_pack.all;

entity controller is
    port (
        Func : in std_logic_vector(NALU-1 downto 0);
        Op : in std_logic_vector(NOPC-1 downto 0);
        ALUControl : out std_logic_vector(NALU-1 downto 0);
        ALUSrc : out std_logic;
        MemtoReg : out std_logic;
        RegWrite : out std_logic;
        RegDst : out std_logic;
        MemWrite : out std_logic;
        Branch : out std_logic
    );
end controller;

architecture Behavioral of controller is

signal op_imm : std_logic_vector(NALU-1 downto 0);
signal func_mod : std_logic_vector(NALU-1 downto 0);

begin

    -- ALUSrc is 0 for R-type, branch or jump; 1 for other instructions
    ALUSrc <= '0' when Op(5 downto 3) = "000" else '1';
    -- MemtoReg is 1 for lw; 0 for other instructions
    MemtoReg <= '1' when Op(5 downto 0) = "100011" else '0';
    -- RegWrite is 1 for R-type, I-type or lw; 0 for other instructions
    RegWrite <= '1' when ((Op(5 downto 0) = "000000" and Func(5 downto 0) /= "000000") or (Op(5 downto 3) = "001" and Op(5 downto 0) /= "001111") or Op(5 downto 0) = "100011") else '0';
    -- MemWrite is 1 for sw; 0 for other instructions
    MemWrite <= '1' when Op(5 downto 0) = "101011" else '0';
    -- RegDst is 0 for I-type and lw; 1 for other instructions
    RegDst <= '0' when (Op(5 downto 3) = "001" or Op(5 downto 0) = "100011") else '1';

    with Op(2 downto 0) select
        op_imm <= "100000" when "000", -- addi
                  "100001" when "001", -- addiu
                  "101000" when "010", -- slti
                  "101001" when "011", -- sltiu
                  "100100" when "100", -- andi
                  "100101" when "101", -- ori
                  "100110" when "110", -- xori
                  (others => '0') when others;

	with Func select
		func_mod <= "101000" when "101010",
					"101001" when "101011",
					Func when others;

    ALUControl <= func_mod when Op(5 downto 0) = "000000" -- R-type
                else op_imm when Op(5 downto 3) = "001" -- I-type
                else "100000" when (Op(5 downto 0) = "100011" or Op(5 downto 0) = "101011") -- lw/sw
                else (others => '0');
                      
    Branch <= '0';
    
end Behavioral;     
