-- regfile.vhd
-- Lab 2
-- Team CART
-- CSE 132L
-- The register file block has 32 registers, each of width 32 bits. It
-- has 2 read ports, and one write port. The read ports operate asynchronously,
-- while the write port operates synchronously. The register file has a
-- write enable port, as well as a synchronous reset input, which if enabled,
-- resets all registers to 0 on the rising edge of the clock.
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.lab2_pack.all;

ENTITY regfile IS

    PORT (
            clk : IN std_logic;
            rst_s : IN std_logic; -- synchronous reset
            we : IN std_logic; -- write enable
            raddr_1 : IN std_logic_vector (NREG-1 DOWNTO 0); -- read address 1
            raddr_2 : IN std_logic_vector (NREG-1 DOWNTO 0); -- read address 2
            waddr : IN std_logic_vector (NREG-1 DOWNTO 0); -- write address
            rdata_1 : OUT std_logic_vector (NBIT-1 DOWNTO 0); -- read data 1
            rdata_2 : OUT std_logic_vector (NBIT-1 DOWNTO 0); -- read data 2
            wdata : IN std_logic_vector (NBIT-1 DOWNTO 0) -- write data 1
    );
END regfile;

architecture Behavioral of regfile is

type reg_array is array ((2**NREG)-1 downto 0) of STD_LOGIC_VECTOR (NBIT-1 downto 0);
signal reg_file : reg_array := (others => (others => '0'));

begin

    rdata_1 <= reg_file(to_integer(unsigned(raddr_1)));
    rdata_2 <= reg_file(to_integer(unsigned(raddr_2)));
    
    Process (clk)
    variable wa : INTEGER := 0;
    begin
        if clk'event and clk = '1' then
            if rst_s = '1' then
                reg_file <= (others => (others => '0'));
            elsif we = '1' then
                wa := to_integer(unsigned(waddr));
                reg_file(wa) <= wdata;
            end if;
        end if;
    end Process;
    
end Behavioral;
