module ram_tb();

	/* local declaration */
	logic clk;
	logic we;
	logic dsgnd;
	logic[1:0] dsize; 
	logic[31:0]	addr;
	logic[31:0]	dataI;

	wire[31:0]	dataO;
  
	/* instantiate a RAM module */
	ram_sim R1(
		.clk(clk),
		.we(we),
		.dsgnd(dsgnd),
		.dsize(dsize),
		.addr(addr),
		.dataI(dataI),
		.dataO(dataO)
	);

always 
begin
	clk = 1'b0;	#5ns;
	clk = 1'b1;	#5ns;	
end

initial 
begin

	/* slight delay to simulate signals finalizing before rising edge */
	#4ns;

	/* read the data at location 0 */
	addr = 32'd0;
	dsgnd = 1'd0;
	dsize = 2'd3;
	we = 1'b0;
	#10ns;

	/* write 1 to location 0 */
	dataI = 32'd1;
	we = 1'b1;
	#10ns;

	/* write 4294967295 to location 1 */
	dataI = 32'd4294967295;
	addr = 32'd4;
	#10ns;

	/* read the data at location 0 */
	we = 1'b0;
	addr = 32'd0;
	#10ns;

	/* read the data at location 1 */
	addr = 32'd4;
	#10ns;	

	/* store 128 in the second-lowest byte of location 1 */
	addr = 32'd4;
	dataI = 32'd128;
	dsize = 2'd0;
	we = 1'd1;
	#10ns;

	/* read the second-lowest byte of location 1, unsigned */
	we = 1'd0;
	#10ns;

	/* read the second-lowest byte of location 1, signed */
	dsgnd = 1'd1;
	#10ns;

	/* read the data in location 1 */
	dsize = 2'd3;
	#10ns;

	/* write 64716 to upper half of location 1 */
	addr = 32'd6;
	dataI = 32'd64716;
	dsize = 2'd1;
	dsgnd = 1'd0;	/* for unsigned output */
	we = 1'd1;
	#10ns;

	/* read the value at the upper half of location 1, unsigned */
	dsgnd = 1'd0;
	we = 1'd0;
	#10ns;

	/* read the value at the upper half of location 1, signed */
	dsgnd = 1'd1;
	#10ns;

	/* read the value at location 1 */
	dsize = 2'd3;
	#10ns;

	$finish;
end

endmodule
    
