library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.lab4_pack.all;

entity controller is
port (
    Instr : in std_logic_vector(NBIT-1 downto 0); -- Current Instrruction
    state : in std_logic_vector(3 downto 0); -- Current state
    Op : out std_logic_vector(2 downto 0);
    IorD : out std_logic;
    ALUSrcA : out std_logic;
    ALUSrcB : out std_logic_vector(1 downto 0);
    ALUControl : out std_logic_vector(NALU-1 downto 0);
    IRWrite : out std_logic;
    PCWrite : out std_logic;
    RegWrite : out std_logic;
    MemWrite : out std_logic;
    Branch : out std_logic;
    RegDst : out std_logic;
    MemtoReg : out std_logic;
    PCSrc : out std_logic_vector(1 downto 0)
);
end controller;

architecture Behavioral of controller is

signal func : std_logic_vector(NALU-1 downto 0);
signal func_mod : std_logic_vector(NALU-1 downto 0);
signal op_imm : std_logic_vector(NALU-1 downto 0);

begin
    IRWrite <= '1' when state = "0000" else '0';
    PCWrite <= '1' when state = "0000" or state = "1011" else '0';
    IorD <= '1' when state = "0011" or state = "0101" else '0';
    RegWrite <= '1' when state = "0100" or state = "0111" or state = "1010"
           else '0'; -- 1 for load, R-type, I-type
    MemWrite <= '1' when state = "0101" else '0'; -- 1 for store
    MemtoReg <= '1' when state = "0100" else '0'; -- 1 for load
    RegDst <= '1' when state = "0111" else '0'; -- 1 for R-type
    Branch <= '1' when state = "1000" else '0'; -- 1 for branch
    ALUSrcA <= '0' when state = "0000" or state = "0001" else '1';
    ALUSrcB <= "01" when state = "0000"
          else "11" when state = "0001"
          else "10" when state = "0010" or state = "1001"
          else "00";
    PCSrc <= "01" when state = "1000"
        else "10" when state = "1011"
        else "00";
    Op <= "000" when Instr(31 downto 26) = "000000" -- R-type
                 and (Instr(5 downto 0) = "000100"
                      or Instr(5 downto 1) = "00011"
                      or Instr(5 downto 3) = "100"
                      or Instr(5 downto 2) = "1010")
     else "001" when Instr(31 downto 29) = "001" -- I-type
                 and Instr(31 downto 26) /= "001111" -- excluding lui
     else "010" when Instr(31 downto 26) = "100011" -- lw
     else "011" when Instr(31 downto 26) = "101011" -- sw
     else "100" when Instr(31 downto 28) = "0001" -- Branch
                  or (Instr(31 downto 26) = "000001"
                      and Instr(20 downto 17) = "0000")
     else "101" when Instr(31 downto 26) = "000010" -- jump
     else "111";

    with Instr(28 downto 26) select
        op_imm <= "100000" when "000", -- addi (add)
                  "100001" when "001", -- addiu (addu)
                  "101000" when "010", -- slti (slt)
                  "101001" when "011", -- sltiu (sltu)
                  "100100" when "100", -- andi (and)
                  "100101" when "101", -- ori (or)
                  "100110" when "110", -- xori (xor)
                  (others => '0') when others;

    -- Reconcile differences in opcodes for slt/sltu between book and lab
    with Instr(5 downto 0) select
        func_mod <= "101000" when "101010",
                    "101001" when "101011",
                    Instr(5 downto 0) when others;

    func <= "111000" when Instr(31 downto 26) = "000001"
                                and Instr(20 downto 16) = "00000" -- bltz
             else "111001" when Instr(31 downto 26) = "000001"
                                and Instr(20 downto 16) = "00001" -- bgez
             else "111100" when Instr(31 downto 26) = "000100" -- beq
             else "111101" when Instr(31 downto 26) = "000101" -- bne
             else "111110" when Instr(31 downto 26) = "000110" -- blez
             else "111111" when Instr(31 downto 26) = "000111" -- bgtz
             else op_imm when Instr(31 downto 29) = "001" -- I-type
             else "100000" when Instr(31 downto 30) = "10" -- load/store
             else func_mod; -- others

    ALUControl <= "100000" when state = "0000" or state = "0001"
             else func;

end Behavioral;
