-- alu.vhd
-- Lab 5
-- Team CART
-- CSE 132L
-- The ALU block can perform arithmetic and logic operations
-- on two operands, each of width 32 bits. It can also compare
-- them for equality, for use with branch instructions.
-- The following operations are supported by this version of the ALU
-- Arith: add, addu, sub, subu, and, or, xor, nor, slt, sltu, lui
-- Branch: beq, bne, bltz, bgez, blez, bgtz
-- Shift: sll, srl, sra, sllv, srlv, srav

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_SIGNED.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.lab5_pack.all;

ENTITY alu IS
    PORT (
        Func_in : IN std_logic_vector (NALU-1 DOWNTO 0);
        A_in : IN std_logic_vector (NBIT-1 DOWNTO 0);
        B_in : IN std_logic_vector (NBIT-1 DOWNTO 0);
        O_out : OUT std_logic_vector (NBIT-1 DOWNTO 0);
        Branch_out : OUT std_logic
    );
END alu;

architecture Behavioral of alu is

signal add_signed, add_unsigned, sub_signed, sub_unsigned, logic_and, logic_or, logic_xor, logic_nor, slt_signed, slt_unsigned, sll_value, srl_value, sra_value : STD_LOGIC_VECTOR (31 downto 0);

begin
	add_signed <= std_logic_vector(signed(A_in) + signed(B_in));
	add_unsigned <= std_logic_vector(unsigned(A_in) + unsigned(B_in));
	sub_signed <= std_logic_vector(signed(A_in) - signed(B_in));
	sub_unsigned <= std_logic_vector(unsigned(A_in) - unsigned(B_in));
    
	logic_and <= A_in AND B_in;
	logic_or <= A_in OR B_in;
	logic_xor <= A_in XOR B_in;
	logic_nor <= A_in NOR B_in;

	
	sll_value <= std_logic_vector(unsigned(B_in) sll to_integer(unsigned(A_in)));
	srl_value <= std_logic_vector(unsigned(B_in) srl to_integer(unsigned(A_in)));
	sra_value <= to_stdlogicvector(to_bitvector(B_in) sra to_integer(unsigned(A_in)));
 
	    slt_signed <= (0 => '1', others => '0') when signed(A_in) < signed(B_in) else (others => '0');
	    slt_unsigned <= (0 => '1', others => '0') when unsigned(A_in) < unsigned(B_in) else (others => '0');
    
	    with Func_in select
	        O_out <= sll_value when "000000" | "000100",
			 srl_value when "000010" | "000110",
			 sra_value when "000011" | "000111",
			 add_signed when "100000",
             add_unsigned when "100001",
             sub_signed when "100010",
             sub_unsigned when "100011",
             logic_and when "100100",
             logic_or when "100101",
             logic_xor when "100110",
             logic_nor when "100111",
             slt_signed when "101000",
             slt_unsigned when "101001",
			 A_in when "111000" | "111001",
			 A_in when "111100" | "111101" | "111110" | "111111",
	                 (others => '0') when others;
                    
	    --Branch_out <= '0';
	    Branch_out <= '1' when (Func_in = "111000" and signed(A_in) < 0)
			        or (Func_in = "111001" and signed(A_in) >= 0)
				or (Func_in = "111100" and signed(A_in) = signed(B_in))
				or (Func_in = "111101" and signed(A_in) /= signed(B_in))
				or (Func_in = "111110" and signed(A_in) <= 0)
				or (Func_in = "111111" and signed(A_in) > 0)
				else '0';
    
end Behavioral;
