library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;
use work.lab4_pack.all;

entity processor is
port (
    ref_clk : in std_logic;
    reset : in std_logic
);
end processor;

architecture Structural of processor is

    signal current_state : std_logic_vector(3 downto 0);
    signal next_state : std_logic_vector(3 downto 0);
    signal Op : std_logic_vector(2 downto 0);
    signal PCEn : std_logic;
    signal IorD : std_logic;
    signal ALUSrcA : std_logic;
    signal ALUSrcB : std_logic_vector(1 downto 0);
    signal ALUControl : std_logic_vector(NALU-1 downto 0);
    signal IRWrite : std_logic;
    signal PCWrite : std_logic;
    signal RegWrite : std_logic;
    signal MemWrite : std_logic;
    signal Branch : std_logic;
    signal RegDst : std_logic;
    signal MemtoReg : std_logic;
    signal PCSrc : std_logic_vector(1 downto 0);

    signal PC : std_logic_vector(NBIT-1 downto 0);
    signal Next_PC : std_logic_vector(NBIT-1 downto 0);
    signal ALUOut : std_logic_vector(NBIT-1 downto 0);
    signal Adr : std_logic_vector(NBIT-1 downto 0);
    signal mem_read : std_logic_Vector(NBIT-1 downto 0); -- instr/data from mem
    signal Instr : std_logic_vector(NBIT-1 downto 0); -- instr read from mem
    signal Data : std_logic_vector(NBIT-1 downto 0); -- data read from memory

    signal waddr : std_logic_vector(NREG-1 downto 0);
    signal wdata : std_logic_vector(NBIT-1 downto 0);

    signal rdata_1 : std_logic_vector(NBIT-1 downto 0);
    signal rdata_2 : std_logic_vector(NBIT-1 downto 0);
    signal dataA : std_logic_vector(NBIT-1 downto 0);
    signal dataB : std_logic_vector(NBIT-1 downto 0);

    signal ImmExt : std_logic_vector(NBIT-1 downto 0);
    signal ImmExtShift : std_logic_vector(NBIT-1 downto 0);

    signal SrcA : std_logic_vector(NBIT-1 downto 0);
    signal SrcB : std_logic_vector(NBIT-1 downto 0);

    signal ALUResult : std_logic_vector(NBIT-1 downto 0);
    signal Zero : std_logic;

    signal jump_imm : std_logic_vector(27 downto 0);
    signal jump_imm_shift : std_logic_vector(27 downto 0);
    signal PCJump : std_logic_vector(NBIT-1 downto 0);
    
    component controller is
    port (
        Instr : in std_logic_vector(NBIT-1 downto 0); -- Current Instrruction
        state : in std_logic_vector(3 downto 0); -- Current state
        Op : out std_logic_vector(2 downto 0);
        IorD : out std_logic;
        ALUSrcA : out std_logic;
        ALUSrcB : out std_logic_vector(1 downto 0);
        ALUControl : out std_logic_vector(NALU-1 downto 0);
        IRWrite : out std_logic;
        PCWrite : out std_logic;
        RegWrite : out std_logic;
        MemWrite : out std_logic;
        Branch : out std_logic;
        RegDst : out std_logic;
        MemtoReg : out std_logic;
        PCSrc : out std_logic_vector(1 downto 0)
    );
    end component;

    component ram is
    port (
        clk :   in std_logic;
        we :    in std_logic;
        addr :  in std_logic_vector (NMEM-1 downto 0);
        dataI : in std_logic_vector (NBIT-1 downto 0);
        dataO : out std_logic_vector (NBIT-1 downto 0)
    );
    end component;

    component regfile is
    port (
        clk : in std_logic;
        rst_s : in std_logic; -- synchronous reset
        we : in std_logic; -- write enable
        raddr_1 : in std_logic_vector (NREG-1 downto 0); -- read address 1
        raddr_2 : in std_logic_vector (NREG-1 downto 0); -- read address 2
        waddr : in std_logic_vector (NREG-1 downto 0); -- write address
        rdata_1 : out std_logic_vector (NBIT-1 downto 0); -- read data 1
        rdata_2 : out std_logic_vector (NBIT-1 downto 0); -- read data 2
        wdata : in std_logic_vector (NBIT-1 downto 0) -- write data 1
    );
    end component;

    component alu is
    port (
        Func_in : in std_logic_vector (NALU-1 downto 0);
        A_in : in std_logic_vector (NBIT-1 downto 0);
        B_in : in std_logic_vector (NBIT-1 downto 0);
        O_out : out std_logic_vector (NBIT-1 downto 0);
        Branch_out : out std_logic
    );
    end component;

    component sign_ext is
    port (
        input : in std_logic_vector(NIMM-1 downto 0);
        output : out std_logic_vector(NBIT-1 downto 0)
    );
    end component;

    component reg is
    generic (
        size : integer := 32
    );
    port (
        clk : in  std_logic;
        en : in std_logic;
        rst : in  std_logic;
        d : in std_logic_vector(size-1 downto 0);
        q : out std_logic_vector(size-1 downto 0)
    );
    end component;

    component mux_2 is
    generic (
        size : integer := 32 -- number of bits for input and output
    );
    port (
        s : in std_logic;
        a : in std_logic_vector(size-1 downto 0);
        b : in std_logic_vector(size-1 downto 0);
        o : out std_logic_vector(size-1 downto 0)   
    );
    end component;

    component mux_4 is
    generic (
        size : integer := 32 -- number of bits for input and output
    );
    port (
        s : in std_logic_vector(1 downto 0);
        a : in std_logic_vector(size-1 downto 0);
        b : in std_logic_vector(size-1 downto 0);
        c : in std_logic_vector(size-1 downto 0);
        d : in std_logic_vector(size-1 downto 0);
        o : out std_logic_vector(size-1 downto 0)   
    );
    end component;

    component shifter is
        generic (
            size : integer := 32;
            amt : integer := 2
        );
        port (
            a : in std_logic_vector(size-1 downto 0);
            o : out std_logic_vector(size-1 downto 0)
        );
    end component;

begin
    PCEn <= (Zero and Branch) or PCWrite;

    jump_imm <= "00" & Instr(25 downto 0);
    PCJump <= Instr(31 downto 28) & jump_imm_shift;

    next_state <= "0001" when current_state = "0000"
             else "0010" when current_state = "0001" and (Op = "010" or Op = "011")
             else "0011" when current_state = "0010" and Op = "010"
             else "0100" when current_state = "0011"
             else "0101" when current_state = "0010" and Op = "011"
             else "0110" when current_state = "0001" and Op = "000"
             else "0111" when current_state = "0110"
             else "1000" when current_state = "0001" and Op = "100"
             else "1001" when current_state = "0001" and Op = "001"
             else "1010" when current_state = "1001"
             else "1011" when current_state = "0001" and Op = "101"
             else "0000";

    control : controller
    port map (
        Instr => Instr,
        state => current_state,
        Op => Op,
        IorD => IorD,
        ALUSrcA => ALUSrcA,
        ALUSrcB => ALUSrcB,
        ALUControl => ALUControl,
        IRWrite => IRWrite,
        PCWrite => PCWrite,
        RegWrite => RegWrite,
        MemWrite => MemWrite,
        Branch => Branch,
        RegDst => RegDst,
        MemtoReg => MemtoReg,
        PCSrc => PCSrc
    );

    state : reg
    generic map (
        size => 4
    )
    port map (
        clk => ref_clk,
        en => '1',
        rst => reset,
        d => next_state,
        q => current_state
    );

    prog_counter : reg
    generic map (
        size => NBIT
    )
    port map (
        clk => ref_clk,
        en => PCEn,
        rst => reset,
        d => Next_PC,
        q => PC
    );

    mem_addr : mux_2
    generic map (
        size => NBIT
    )
    port map (
        s => IorD,
        a => PC,
        b => ALUOut,
        o => Adr
    );

    memory : ram
    port map (
        clk => ref_clk,
        we => MemWrite,
        addr => Adr(NMEM-1 downto 0),
        dataI => dataB,
        dataO => mem_read
    );

    instr_reg : reg
    generic map (
        size => NBIT
    )
    port map (
        clk => ref_clk,
        en => IRWrite,
        rst => reset,
        d => mem_read,
        q => Instr
    );

    mem_data_reg : reg
    generic map (
        size => NBIT
    )
    port map (
        clk => ref_clk,
        en => '1',
        rst => reset,
        d => mem_read,
        q => Data
    );

    reg_write_addr : mux_2
    generic map (
        size => NREG
    )
    port map (
        s => RegDst,
        a => Instr(20 downto 16),
        b => Instr(15 downto 11),
        o => waddr
    );

    reg_write_data : mux_2
    generic map (
        size => NBIT
    )
    port map (
        s => MemtoReg,
        a => ALUOut,
        b => Data,
        o => wdata
    );

    register_file : regfile
    port map (
        clk => ref_clk,
        rst_s => reset,
        we => RegWrite,
        raddr_1 => Instr(25 downto 21),
        raddr_2 => Instr(20 downto 16),
        waddr => waddr,
        rdata_1 => rdata_1,
        rdata_2 => rdata_2,
        wdata => wdata
    );

    extend_imm : sign_ext
    port map (
        input => Instr(15 downto 0),
        output => ImmExt
    );

    rdata_1_reg : reg
    generic map (
        size => NBIT
    )
    port map (
        clk => ref_clk,
        en => '1',
        rst => reset,
        d => rdata_1,
        q => dataA
    );

    rdata_2_reg : reg
    generic map (
        size => NBIT
    )
    port map (
        clk => ref_clk,
        en => '1',
        rst => reset,
        d => rdata_2,
        q => dataB
    );
    alu_src_a : mux_2
    generic map (
        size => NBIT
    )
    port map (
        s => ALUSrcA,
        a => PC,
        b => dataA,
        o => SrcA
    );

    alu_src_b : mux_4
    generic map (
        size => NBIT
    )
    port map (
        s => ALUSrcB,
        a => dataB,
        b => "00000000000000000000000000000100",
        c => ImmExt,
        d => ImmExtShift,
        o => SrcB
    );

    shift_imm : shifter
    generic map (
        size => NBIT,
        amt => 2
    )
    port map (
        a => ImmExt,
        o => ImmExtShift
    );

    alu_block : alu
    port map (
        Func_in => ALUControl,
        A_in => SrcA,
        B_in => SrcB,
        O_out => ALUResult,
        Branch_out => Zero
    );

    alu_result_reg : reg
    generic map (
        size => NBIT
    )
    port map (
        clk => ref_clk,
        en => '1',
        rst => reset,
        d => ALUResult,
        q => ALUOut
    );

    shift_jump_imm : shifter
    generic map (
        size => 28,
        amt => 2
    )
    port map (
        a => jump_imm,
        o => jump_imm_shift
    );

    pc_src : mux_4
    generic map (
        size => NBIT
    )
    port map (
        s => PCSrc,
        a => ALUResult,
        b => ALUOut,
        c => PCJump,
        d => (others => '0'),
        o => Next_PC
    );

end Structural;
