-- ram.vhd
-- Lab 4
-- Team CART
-- CSE 132L
-- The RAM block is used for the data memory. It has an address port
-- for the read and write addresses, a port for read data, a port for
-- write data, a write enable signal, and a clock input. The write
-- operates synchronously, while the read operates asynchronously.
-- The RAM is of size 512x32 bits.
library ieee;
USE ieee.numeric_std.all;
USE ieee.std_logic_1164.all;
use work.lab4_pack.all;
use IEEE.NUMERIC_STD_UNSIGNED.ALL;
use STD.TEXTIO.ALL;

entity ram is
port (
    clk :   in std_logic;
    we :    in std_logic;
    addr :  in std_logic_vector (NMEM-1 downto 0);
    dataI : in std_logic_vector (NBIT-1 downto 0);
    dataO : out std_logic_vector (NBIT-1 downto 0)
);
end ram;

architecture Behavioral of ram is

    type mem_array is array ((2**NMEM)-1 downto 0) of std_logic_vector (NBIT-1 downto 0);
    signal memory : mem_array;

begin
    process is
    
    FILE f : TEXT;
    variable L : line;
    variable ch : character;
    variable i, index, result : integer := 0;
    TYPE mem IS ARRAY (2**(NBIT/2)-1 DOWNTO 0) OF std_logic_vector (NBIT-1 DOWNTO 0);
    variable memory : mem;
    
    begin
    
        for i in 0 to 2**(NBIT/2)-1 loop
            memory(i) := (others => '0');
        end loop;
    
        FILE_OPEN(f, "rom.dat", READ_MODE);
        while not endfile(f) loop
            readline(f, L);
            
            for i in 1 to 8 loop
                read(L, ch);
                if '0' <= ch and ch <= '9' then
                    result := character'pos(ch) - character'pos('0');
                elsif 'a' <= ch and ch <= 'f' then
                    result := character'pos(ch) - character'pos('a')+10;
                else report "Format error on line" & integer'
                    image(index) severity error;
                end if;
                memory(index)(35-i*4 DOWNTO 32-i*4) := to_std_logic_vector(result, 4);
            end loop;
            index := index + 1;
        end loop;
        
        loop 
            wait on addr, clk;
            if (clk'event and clk = '1' and we = '1') then
                -- If write enabled is HIGH, write input to memory 
                memory(to_integer((addr(NMEM-1 downto 2)))) := dataI;
            end if;
            if (addr'event) then
                dataO <= memory(to_integer(addr(NMEM-1 downto 2)));
            end if;
        end loop;
        
    end process;

--    memory(0) <= "00100000000000011111111111111111";
--    memory(1) <= "00100100000000011111111111111111";
--    memory(2) <= "00101000000000011111111111111111";
--    memory(3) <= "00101100000000011111111111111111";
--    memory(4) <= "00100000000000100101010101010101";
--    memory(5) <= "00110000010000111111111100000000";
--    memory(6) <= "00110100010000111111111100000000";
--    memory(7) <= "00111000010000111111111100000000";
--    memory(8) <= "00100000000000010000000011111111";
--    memory(9) <= "00100000000000101111111100000000";
--    memory(10) <= "00000000001000100001100000100000";
--    memory(11) <= "00000000001000100001100000100001";
--    memory(12) <= "00000000001000100001100000100010";
--    memory(13) <= "00000000001000100001100000100011";
--    memory(14) <= "00100000000000010101010101010101";
--    memory(15) <= "00100000000000101111111100000000";
--    memory(16) <= "00000000001000100001100000100100";
--    memory(17) <= "00000000001000100001100000100101";
--    memory(18) <= "00000000001000100001100000100110";
--    memory(19) <= "00000000001000100001100000100111";

--    do_ram_things: process(clk)
--    begin
--        -- If the clock has changed state and is on the rising edge
--        if (clk = '1') then
--            -- If write enabled is HIGH, write input to memory 
--            if (we = '1') then
--                memory(to_integer(unsigned(addr))) <= dataI;
--            end if; 
--        end if;    
--
--    end process;
--    
--    -- Read data from memory
--    dataO <= memory(to_integer(unsigned(addr(NMEM-1 downto 2))));

end Behavioral;

