-- ram.vhd
-- Lab 3
-- Team CART
-- CSE 132L
-- The RAM block is used for the data memory. It has an address port
-- for the read and write addresses, a port for read data, a port for
-- write data, a write enable signal, and a clock input. The write
-- operates synchronously, while the read operates asynchronously.
-- The RAM is of size 512x32 bits.

LIBRARY ieee;
USE ieee.numeric_std.ALL;
USE ieee.std_logic_1164.ALL;
USE work.lab3_pack.ALL;

ENTITY ram IS
	PORT (
		clk : 	IN std_logic;
		we : 	IN std_logic;
		dsgnd : IN std_logic;
		dsize : IN std_logic_vector(1 DOWNTO 0);
		addr : 	IN std_logic_vector (NBIT - 1 DOWNTO 0);
		dataI : IN std_logic_vector (NBIT - 1 DOWNTO 0);
		dataO : OUT std_logic_vector (NBIT - 1 DOWNTO 0)
	);
END ram;

ARCHITECTURE Behavioral OF ram IS

	TYPE mem_array IS ARRAY (2**NMEM - 1 DOWNTO 0) OF std_logic_vector (NBIT-1 DOWNTO 0);
	SIGNAL memory : mem_array := (others => (others => '0'));

	SIGNAL word_addr : integer RANGE 0 to 2**NMEM - 1;
	SIGNAL half_addr : integer RANGE 0 to 1;
	SIGNAL byte_addr : integer RANGE 0 to 3;

BEGIN

	word_addr <= to_integer(unsigned(addr(NMEM+2-1 DOWNTO 2)));
	half_addr <= to_integer(unsigned(addr(1 DOWNTO 1)));
	byte_addr <= to_integer(unsigned(addr(1 DOWNTO 0)));

	do_ram_things: PROCESS(clk)
		VARIABLE word_out : std_logic_vector (NBIT - 1 DOWNTO 0);
		VARIABLE half_out : std_logic_vector (NBIT / 2 DOWNTO 0);
		VARIABLE byte_out : std_logic_vector (NBIT / 4 DOWNTO 0);
	BEGIN
		IF (rising_edge(clk) and we = '1') THEN
			IF (dsize = "00") THEN
				memory(word_addr)(byte_addr * 8 + 7 DOWNTO byte_addr * 8) <= dataI(7 DOWNTO 0);
			ELSIF (dsize = "01") THEN
				memory(word_addr)(half_addr * 16 + 15 DOWNTO half_addr * 16) <= dataI(15 DOWNTO 0);
			ELSE
				memory(word_addr) <= dataI;
			END IF;
		END IF;

		IF falling_edge(clk) THEN
			word_out := memory(word_addr);
			
			IF (dsgnd = '0') THEN
				half_out := '0' & memory(word_addr)(half_addr * 16 + 15 DOWNTO half_addr * 16);
				byte_out := '0' & memory(word_addr)(byte_addr * 8 + 7 DOWNTO byte_addr * 8);
			ELSE 
				half_out := memory(word_addr)(half_addr * 16 + 15) & memory(word_addr)(half_addr * 16 + 15 DOWNTO half_addr * 16);
				byte_out := memory(word_addr)(half_addr * 8 + 7) & memory(word_addr)(byte_addr * 8 + 7 DOWNTO byte_addr * 8);
			END IF;

			IF (dsize = "00") THEN
				dataO <= (31 DOWNTO 8 => byte_out(8)) & byte_out(7 DOWNTO 0);
			ELSIF (dsize = "01") THEN
				dataO <= (31 DOWNTO 16 => half_out(16)) & half_out (15 DOWNTO 0);
			ELSE
				dataO <= word_out;
			END IF;
		END IF;
	END PROCESS;	

END Behavioral;

