-- rom.vhd
-- Lab 5
-- Team CART
-- CSE 132L
-- The ROM component is used for the instrution memory. It has an input port
-- for the address of the instruction to be fetched, and an inout port for
-- the instruction. Here, we use an inout port to be able to preload the
-- instruction memory before reading the instructions. The size of the ROM
-- is 512x32 bits. Initially, we tried using 2^32 lines for the ROM, but
-- we found that both Cadence Incisive and MentorGraphics QuestaSim had
-- issues with a ROM block of that large a size, so we decided to reduce
-- the number of rows to 512, since that would be sufficient for our
-- testing purposes.

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use IEEE.NUMERIC_STD_UNSIGNED.ALL;
use STD.TEXTIO.ALL;

library work;
use work.lab5_pack.all;

ENTITY rom IS
	PORT
	(
		addr   : IN std_logic_vector(NBIT-1 DOWNTO 0);
		dataIO : INOUT std_logic_vector(NBIT-1 DOWNTO 0)
	);
END rom;

ARCHITECTURE Behavorial OF rom IS
begin
	process is
	
	FILE f : TEXT;
	variable L : line;
	variable ch : character;
	variable i, index, result : integer := 0;
	TYPE mem IS ARRAY (2**(NBIT/2)-1 DOWNTO 0) OF std_logic_vector (NBIT-1 DOWNTO 0);
	variable instr_mem : mem;
	
	begin
	
		for i in 0 to 2**(NBIT/2)-1 loop
			instr_mem(i) := (others => '0');
		end loop;
	
		FILE_OPEN(f, "rom.dat", READ_MODE);
		while not endfile(f) loop
			readline(f, L);
			
			for i in 1 to 8 loop
				read(L, ch);
				if '0' <= ch and ch <= '9' then
					result := character'pos(ch) - character'pos('0');
				elsif 'a' <= ch and ch <= 'f' then
					result := character'pos(ch) - character'pos('a')+10;
				else report "Format error on line" & integer'
					image(index) severity error;
				end if;
				instr_mem(index)(35-i*4 DOWNTO 32-i*4) := to_std_logic_vector(result, 4);
			end loop;
			index := index + 1;
		end loop;
		
		loop 
			wait on addr;
			dataIO <= instr_mem(to_integer(addr(NMEM+2-1 downto 2)));
		end loop;
		
	end process;

end Behavorial;
