library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;
use work.lab5_pack.all;

entity stage_writeback is
port (
    clk : in std_logic;
    rst :in std_logic;
    RegWriteM: in std_logic;
    MemtoRegM : in std_logic;
    ALUOutM : in std_logic_vector(NBIT-1 downto 0);
    ReadDataM : in std_logic_vector(NBIT-1 downto 0);
    WriteRegM : in std_logic_vector(NREG-1 downto 0);
    PCSrcD : in std_logic;
	PCJumpD : in std_logic;
	PCJumpAddr : in std_logic_vector(NBIT-1 downto 0);
    PCBranchD : in std_logic_vector(NBIT-1 downto 0);
    PCPlus4F : in std_logic_vector(NBIT-1 downto 0);
    RegWriteW : out std_logic;
    WriteRegW: out std_logic_vector(NREG-1 downto 0);
    ResultW : out std_logic_vector(NBIT-1 downto 0);
    PC_p : out std_logic_vector(NBIT-1 downto 0)

);
end stage_writeback;

architecture Structural of stage_writeback is


    component memory_writeback_reg is
    port (
        clk : in std_logic;
        rst : in std_logic;
        RegWriteM : in std_logic;
        MemtoRegM : in std_logic;
        ALUOutM : in std_logic_vector(NBIT-1 downto 0);
        ReadDataM : in std_logic_vector(NBIT-1 downto 0);
        WriteRegM : in std_logic_vector(NREG-1 downto 0);
        RegWriteW : out std_logic;
        MemtoRegW : out std_logic;
        ALUOutW : out std_logic_vector(NBIT-1 downto 0);
        ReadDataW : out std_logic_vector(NBIT-1 downto 0);
        WriteRegW : out std_logic_vector(NREG-1 downto 0)
    );
    end component;



  COMPONENT mux IS
	GENERIC (
        	NBIT :	integer := 1	-- number of bits in each input
	);
	PORT (
		s :	IN std_logic;
		a :	IN std_logic_vector(NBIT-1 DOWNTO 0);
		b :	IN std_logic_vector(NBIT-1 DOWNTO 0);
		o :	OUT std_logic_vector(NBIT-1 DOWNTO 0)
	);
	END COMPONENT;

  signal MemToRegW : std_logic;
  signal ALUOutW : std_logic_vector(NBIT-1 downto 0);
  signal ReadDataW : std_logic_vector(NBIT-1 downto 0);
  signal PCAddrTemp : std_logic_vector(NBIT-1 downto 0);

begin

    memory_writeback_pipeline : memory_writeback_reg
    port map (
    clk => clk,
    rst => rst,
    RegWriteM => RegWriteM,
    MemtoRegM => MemtoRegM,
    ALUOutM => ALUOutM,
    ReadDataM => ReadDataM,
    WriteRegM => WriteRegM,
    RegWriteW => RegWriteW,
    MemtoRegW => MemtoRegW,
    ALUOutW => ALUOutW,
    ReadDataW => ReadDataW,
    WriteRegW => WriteRegW

    );

    result_mux : mux
    GENERIC MAP(NBIT => 32)
    PORT MAP(
    	s => MemtoRegW,
    	a => ALUOutW,
    	b => ReadDataW,
    	o => ResultW
    );


    pc_branch_mux : mux
	GENERIC MAP(NBIT => 32)
	PORT MAP(
      	s => PCSrcD,
      	a => PCPlus4F,
      	b => PCBranchD,
      	o => PCAddrTemp
	);

	pc_jump_mux : mux
	GENERIC MAP (NBIT => 32)
	PORT MAP (
		s => PCJumpD,
		a => PCAddrTemp,
		b => PCJumpAddr,
		o => PC_p 
	);

end Structural;
