LIBRARY IEEE;
USE ieee.numeric_std.ALL;
USE ieee.std_logic_1164.ALL;
USE work.lab5_pack.ALL;

ENTITY decode_execute_reg IS
PORT (
	clk :   IN std_logic;
	rst :   IN std_logic;
	
	ALUSrcIn :   IN std_logic;
	FlushE :     IN std_logic;
	MemtoRegIn : IN std_logic;
	MemWriteIn : IN std_logic;
	RegDstIn :   IN std_logic;
	RegWriteIn : IN std_logic;
	
	ALUControlIn : IN std_logic_vector (NALU-1 DOWNTO 0);
	Data1In :      IN std_logic_vector (NBIT-1 DOWNTO 0);
	Data2In :      IN std_logic_vector (NBIT-1 DOWNTO 0);
	RdIn :         IN std_logic_vector (NREG-1 DOWNTO 0);
	RsIn :         IN std_logic_vector (NREG-1 DOWNTO 0);
	RtIn :         IN std_logic_vector (NREG-1 DOWNTO 0);
	SignImmIn :    IN std_logic_vector (NBIT-1 DOWNTO 0);
	
	ALUSrcE :   OUT std_logic;
	MemtoRegE : OUT std_logic;
	MemWriteE : OUT std_logic;
	RegDstE :   OUT std_logic;
	RegWriteE : OUT std_logic;
	
	ALUControlE : OUT std_logic_vector (NALU-1 DOWNTO 0);
	Data1E :      OUT std_logic_vector (NBIT-1 DOWNTO 0);
	Data2E :      OUT std_logic_vector (NBIT-1 DOWNTO 0);
	RdE :         OUT std_logic_vector (NREG-1 DOWNTO 0);
	RsE :         OUT std_logic_vector (NREG-1 DOWNTO 0);
	RtE :         OUT std_logic_vector (NREG-1 DOWNTO 0);
	SignImmE :    OUT std_logic_vector (NBIT-1 DOWNTO 0)
);
END decode_execute_reg;

ARCHITECTURE Behavorial OF decode_execute_reg IS
BEGIN
	PROCESS(clk)
	BEGIN
		IF (clk'event and clk = '1') THEN
			IF (rst = '1' or FlushE = '1') THEN
				ALUSrcE   <= '0';
				MemtoRegE <= '0';
				MemWriteE <= '0';
				RegDstE   <= '0';
				RegWriteE <= '0';
			
				ALUControlE <= (OTHERS => '0');
				Data1E      <= (OTHERS => '0');
				Data2E      <= (OTHERS => '0');
				RdE         <= (OTHERS => '0');
				RsE         <= (OTHERS => '0');
				RtE         <= (OTHERS => '0');
				SignImmE    <= (OTHERS => '0');
			ELSE
				ALUSrcE   <= ALUSrcIn;
				MemtoRegE <= MemtoRegIN;
				MemWriteE <= MemWriteIn;
				RegDstE   <= RegDstIn;
				RegWriteE <= RegWriteIn;
				
				ALUControlE <= ALUControlIn;
				Data1E      <= Data1In;
				Data2E      <= Data2In;
				RdE         <= RdIn;
				RsE         <= RsIn;
				RtE         <= RtIn;
				SignImmE    <= SignImmIn;
			END IF;
		END IF;
	END PROCESS;
END Behavorial;

