LIBRARY IEEE;
USE ieee.numeric_std.ALL;
USE ieee.std_logic_1164.ALL;
USE work.lab5_pack.ALL;

ENTITY stage_execute IS
PORT (
	clk :   IN std_logic;
	rst :   IN std_logic;

	ALUSrcIn :   IN std_logic;
	FlushE :     IN std_logic;
	MemtoRegIn : IN std_logic;
	MemWriteIn : IN std_logic;
	RegDstIn :   IN std_logic;
	RegWriteIn : IN std_logic;
	
	ALUControlIn : IN std_logic_vector (NALU-1 DOWNTO 0);
	Data1In :      IN std_logic_vector (NBIT-1 DOWNTO 0);
	Data2In :      IN std_logic_vector (NBIT-1 DOWNTO 0);
	ForwardAE :    IN std_logic_vector (1      DOWNTO 0);
	ForwardBE :    IN std_logic_vector (1      DOWNTO 0);
	ForwardDataM : IN std_logic_vector (NBIT-1 DOWNTO 0);
	ForwardDataW : IN std_logic_vector (NBIT-1 DOWNTO 0);
	RdIn :         IN std_logic_vector (NREG-1 DOWNTO 0);
	RsIn :         IN std_logic_vector (NREG-1 DOWNTO 0);
	RtIn :         IN std_logic_vector (NREG-1 DOWNTO 0);
	SignImmIn :    IN std_logic_vector (NBIT-1 DOWNTO 0);
	
	MemtoRegOut : OUT std_logic;
	MemWriteOut : OUT std_logic;
	RegWriteOut : OUT std_logic;
	
	ALUOut :       OUT std_logic_vector (NBIT-1 DOWNTO 0);
	RsOut :        OUT std_logic_vector (NREG-1 DOWNTO 0);
	RtOut :        OUT std_logic_vector (NREG-1 DOWNTO 0);
	WriteDataOut : OUT std_logic_vector (NBIT-1 DOWNTO 0);
	WriteRegOut :  OUT std_logic_vector (NREG-1 DOWNTO 0)
);
END stage_execute;

ARCHITECTURE Behavioral OF stage_execute IS

	-- COMPONENT DECLARATION
	
	COMPONENT decode_execute_reg IS
	PORT (
		clk :   IN std_logic;
		rst :   IN std_logic;
		
		ALUSrcIn :   IN std_logic;
		FlushE :     IN std_logic;
		MemtoRegIn : IN std_logic;
		MemWriteIn : IN std_logic;
		RegDstIn :   IN std_logic;
		RegWriteIn : IN std_logic;
		
		ALUControlIn : IN std_logic_vector (NALU-1 DOWNTO 0);
		Data1In :      IN std_logic_vector (NBIT-1 DOWNTO 0);
		Data2In :      IN std_logic_vector (NBIT-1 DOWNTO 0);
		RdIn:          IN std_logic_vector (NREG-1 DOWNTO 0);
		RsIn :         IN std_logic_vector (NREG-1 DOWNTO 0);
		RtIn:          IN std_logic_vector (NREG-1 DOWNTO 0);
		SignImmIn :    IN std_logic_vector (NBIT-1 DOWNTO 0);
		
		ALUSrcE :   OUT std_logic;
		MemtoRegE : OUT std_logic;
		MemWriteE : OUT std_logic;
		RegDstE :   OUT std_logic;
		RegWriteE : OUT std_logic;
		
		ALUControlE : OUT std_logic_vector (NALU-1 DOWNTO 0);
		Data1E :      OUT std_logic_vector (NBIT-1 DOWNTO 0);
		Data2E :      OUT std_logic_vector (NBIT-1 DOWNTO 0);
		RdE:          OUT std_logic_vector (NREG-1 DOWNTO 0);
		RsE :         OUT std_logic_vector (NREG-1 DOWNTO 0);
		RtE:          OUT std_logic_vector (NREG-1 DOWNTO 0);
		SignImmE :    OUT std_logic_vector (NBIT-1 DOWNTO 0)
	);
	END COMPONENT;

	COMPONENT alu IS
	PORT (
		Func_in :    IN std_logic_vector  (NALU-1 DOWNTO 0);
		A_in :       IN std_logic_vector  (NBIT-1 DOWNTO 0);
		B_in :       IN std_logic_vector  (NBIT-1 DOWNTO 0);
		O_out :      OUT std_logic_vector (NBIT-1 DOWNTO 0);
		Branch_out : OUT std_logic
	);
	END COMPONENT;
	
	COMPONENT mux IS
	GENERIC(
		NBIT : integer := 32
	);
	PORT (
		s : IN  std_logic;
		a :	IN  std_logic_vector (NBIT-1 DOWNTO 0);
		b :	IN  std_logic_vector (NBIT-1 DOWNTO 0);
		o :	OUT std_logic_vector (NBIT-1 DOWNTO 0) 	
	);
	END COMPONENT;
	
	COMPONENT mux3 IS
	PORT (
		s : IN  std_logic_vector (1      DOWNTO 0);
		a :	IN  std_logic_vector (NBIT-1 DOWNTO 0);
		b :	IN  std_logic_vector (NBIT-1 DOWNTO 0);
		c : IN  std_logic_vector (NBIT-1 DOWNTO 0);
		o :	OUT std_logic_vector (NBIT-1 DOWNTO 0) 	
	);
	END COMPONENT;

	-- SIGNAL DECLARATION

	SIGNAL ALUSrcE :   std_logic;
	SIGNAL DummyZero : std_logic;
	SIGNAL MemtoRegE : std_logic;
	SIGNAL MemWriteE : std_logic;
	SIGNAL RegDstE :   std_logic;
	SIGNAL RegWriteE : std_logic;
	
	SIGNAL ALUControlE : std_logic_vector (NALU-1 DOWNTO 0);
	SIGNAL Data1E :      std_logic_vector (NBIT-1 DOWNTO 0);
	SIGNAL Data2E :      std_logic_vector (NBIT-1 DOWNTO 0);
	SIGNAL RdE :         std_logic_vector (NREG-1 DOWNTO 0) := (OTHERS => '0');
	SIGNAL RsE :         std_logic_vector (NREG-1 DOWNTO 0) := (OTHERS => '0');
	SIGNAL RtE :         std_logic_vector (NREG-1 DOWNTO 0) := (OTHERS => '0');
	SIGNAL SignImmE :    std_logic_vector (NBIT-1 DOWNTO 0) := (OTHERS => '0');
	SIGNAL SrcAE :       std_logic_vector (NBIT-1 DOWNTO 0) := (OTHERS => '0');
	SIGNAL SrcBE :       std_logic_vector (NBIT-1 DOWNTO 0) := (OTHERS => '0');
	SIGNAL WriteDataE :  std_logic_vector (NBIT-1 DOWNTO 0) := (OTHERS => '0');

BEGIN

	-- SIGNAL ASSIGNMENT
	
    MemtoRegOut <= MemtoRegE;
    MemWriteOut <= MemWriteE;
    RegWriteOut <= RegWriteE;

	RsOut        <= RsE;
	RtOut        <= RtE;
	WriteDataOut <= WriteDataE;

	-- COMPONENT INSTANTIATION
	
	reg : decode_execute_reg
	PORT MAP (
		clk =>   clk,
		rst =>   rst,
			
		ALUSrcIn   => ALUSrcIn,
		FlushE     => FlushE,
		MemtoRegIn => MemToRegIn,
		MemWriteIn => MemWriteIn,
		RegDstIn   => RegDstIn,
		RegWriteIn => RegWriteIn,
		
		ALUControlIn => ALUControlIn,
		Data1In      => Data1In,
		Data2In      => Data2In,
		RdIn         => RdIn,
		RsIn         => RsIn,
		RtIn         => RtIn,
		SignImmIn    => SignImmIn,
		
		ALUSrcE   => ALUSrcE,
		MemtoRegE => MemtoRegE,
		MemWriteE => MemWriteE,
		RegDstE   => RegDstE,
		RegWriteE => RegWriteE,
		
		ALUControlE => ALUControlE,
		Data1E      => Data1E,
		Data2E      => Data2E,
		RdE         => RdE,  
		RsE         => RsE,  
		RtE         => RtE,       
		SignImmE    => SignImmE 
	);
	
	datapath_alu : alu
	PORT MAP (
		Func_in    => ALUControlE,
		A_in       => SrcAE,
		B_in       => SrcBE,
		O_out      => ALUOut,
		Branch_out => DummyZero
	);
	
	datapath_mux2_alu_b : mux
	GENERIC MAP (
		NBIT => NBIT
	)
	PORT MAP (
		s => ALUSrcE,
		a => WriteDataE,
		b => SignImmE,
		o => SrcBE
	);
	
	datapath_mux3_alu_a : mux3
	PORT MAP (
		s => ForwardAE,
		a => Data1E,
		b => ForwardDataW,
		c => ForwardDataM,
		o => SrcAE
	);

	datapath_mux3_alu_b : mux3
	PORT MAP (
		s => ForwardBE,
		a => Data2E,
		b => ForwardDataW,
		c => ForwardDataM,
		o => WriteDataE
	);

	datapath_mux_reg : mux
	GENERIC MAP (
		NBIT => NREG
	)
	PORT MAP (
		s => RegDstE,
		a => RtE,
		b => RdE,
		o => WriteRegOut
	);
	
END Behavioral;
