library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;
use work.lab5_pack.all;

entity stage_fetch is
port (
    clk : in std_logic;
    rst : in std_logic;
    StallF : in std_logic;
    PC_p: in std_logic_vector(NBIT-1 downto 0);
    InstrF : out std_logic_vector(NBIT-1 downto 0);
    PCPlus4F : out std_logic_vector(NBIT-1 downto 0)

);
end stage_fetch;

architecture Structural of stage_fetch is

    COMPONENT writeback_fetch_reg is
    port (
        clk : in std_logic;
        rst : in std_logic;
        en : in std_logic;
        PC_p : in STD_LOGIC_VECTOR (NBIT-1 downto 0);
        PCF : out std_logic_vector(NBIT-1 downto 0)
    );
    END COMPONENT;

    COMPONENT adder IS
	PORT (
		a :	IN std_logic_vector(NBIT-1 DOWNTO 0);
		b : IN std_logic_vector(NBIT-1 DOWNTO 0);
		c : OUT std_logic_vector(NBIT-1 DOWNTO 0)
	);
    END COMPONENT;

    COMPONENT rom IS
        PORT (
        	addr   : IN std_logic_vector(NBIT-1 DOWNTO 0);
        	dataIO : INOUT std_logic_vector(NBIT-1 DOWNTO 0)
        );
	END COMPONENT;

  COMPONENT mux IS
	GENERIC (
        	NBIT :	integer := 1	-- number of bits in each input
	);
	PORT (
		s :	IN std_logic;
		a :	IN std_logic_vector(NBIT-1 DOWNTO 0);
		b :	IN std_logic_vector(NBIT-1 DOWNTO 0);
		o :	OUT std_logic_vector(NBIT-1 DOWNTO 0)
	);
	END COMPONENT;


  signal PCF : std_logic_vector(NBIT-1 DOWNTO 0);
  signal dataIO_buf : std_logic_vector(NBIT-1 DOWNTO 0);
  signal not_StallF : std_logic;

begin

writeback_fetch_pipeline : writeback_fetch_reg
port map (
clk => clk,
rst => rst,
en => not_StallF,
PC_p => PC_p,
PCF => PCF
);

	increment_pc : adder
	PORT MAP(
		a => PCF,
		b => (2 => '1', others => '0'),
		c => PCPlus4F
	);



	instr_mem : rom
	PORT MAP (
	        addr => PCF,
	        dataIO => dataIO_buf
	);

    InstrF <= dataIO_buf;
    not_StallF <= not StallF;


---
end Structural;
