library IEEE;
use ieee.numeric_std.ALL;
use ieee.std_logic_1164.ALL;
use work.lab5_pack.ALL;

ENTITY stage_memory IS
PORT (
	clk : IN std_logic;
	rst : IN std_logic;
	
	RegWriteE : IN std_logic;
	MemtoRegE : IN std_logic;
	MemWriteE : IN std_logic;
	ALUOutE : IN std_logic_vector(NBIT-1 DOWNTO 0);
	WriteDataE : IN std_logic_vector(NBIT-1 DOWNTO 0);
	WriteRegE : IN std_logic_vector(NREG-1 DOWNTO 0);
	
	RegWriteM : OUT std_logic;
	MemtoRegM : OUT std_logic;
	ALUOutM : OUT std_logic_vector(NBIT-1 DOWNTO 0);
	WriteRegM : OUT std_logic_vector(NREG-1 DOWNTO 0);
	ReadDataM : OUT std_logic_vector(NBIT-1 DOWNTO 0);
	WriteRegM_hazard : OUT std_logic_vector(NREG-1 DOWNTO 0);
	ALUOutM_execute : OUT std_logic_vector(NBIT-1 DOWNTO 0)
);
END stage_memory;

ARCHITECTURE Structural OF stage_memory IS

	signal t_MemWriteM : std_logic;
	signal t_ALUOutM : std_logic_vector(NBIT-1 DOWNTO 0);
	signal t_WriteDataM: std_logic_vector(NBIT-1 DOWNTO 0);
	signal t_BranchM : std_logic;
	signal t_WriteRegM : std_logic_vector(NREG-1 DOWNTO 0);

	COMPONENT execute_memory_reg IS
	PORT(
		clk : IN std_logic;
		rst : IN std_logic;
		
		RegWriteE : IN std_logic;
		MemtoRegE : IN std_logic;
		MemWriteE : IN std_logic;
		ALUOutE : IN std_logic_vector(NBIT-1 DOWNTO 0);
		WriteDataE : IN std_logic_vector(NBIT-1 DOWNTO 0);
		WriteRegE : IN std_logic_vector(NREG-1 DOWNTO 0);
		
		RegWriteM : OUT std_logic;
		MemtoRegM : OUT std_logic;
		MemWriteM : OUT std_logic;
		ALUOutM : OUT std_logic_vector(NBIT-1 DOWNTO 0);
		WriteDataM : OUT std_logic_vector(NBIT-1 DOWNTO 0);
		WriteRegM : OUT std_logic_vector(NREG-1 DOWNTO 0)
	);
	END COMPONENT;

--	COMPONENT ram IS
--	PORT (
--		clk : IN std_logic;
--		we : IN std_logic;
--		addr : IN std_logic_vector(NMEM-1 DOWNTO 0);
--		dataI : IN std_logic_vector(NBIT-1 DOWNTO 0);
--		dataO : OUT std_logic_vector(NBIT-1 DOWNTO 0)
--	);
--	END COMPONENT;

	COMPONENT dmem PORT (
		clk : in std_logic;
		we : in std_logic;
		a : in std_logic_vector(31 downto 0);
		wd : in std_logic_vector(31 downto 0);
		rd : out std_logic_vector(31 downto 0)
	);
	END COMPONENT;

BEGIN

	execute_memory_pipeline : execute_memory_reg
	PORT MAP (
		clk => clk,
		rst => rst,
		
		RegWriteE => RegWriteE,
		MemtoRegE => MemtoRegE,
		MemWriteE => MemwriteE,
		ALUOutE => ALUOutE,
		WriteDataE => WriteDataE,
		WriteRegE => WriteRegE,

		RegWriteM => RegWriteM,
		MemtoRegM => MemtoRegM,
		MemWriteM => t_MemWriteM,
		ALUOutM => t_ALUOutM,
		WriteDataM => t_WriteDataM,
		WriteRegM => t_WriteRegM
	);

--	data_memory : ram 
--	PORT MAP (
--		clk => clk,
--		we => t_MemWriteM,
--		addr => t_ALUOutM(NMEM-1 downto 0),
--		dataI => t_WriteDataM,
--		dataO => ReadDataM
--	);

	datapath_ram : dmem
	PORT MAP (
		clk   => clk,
		we    => t_MemWriteM,
		a  => t_ALUOutM,
		wd => t_WriteDataM,
		rd => ReadDataM
	);
	
    ALUOutM <= t_ALUOutM;
    WriteRegM_hazard <= t_WriteRegM;
    ALUOutM_execute <= t_ALUOutM;
    WriteRegM <= t_WriteRegM;

END	Structural;
